/*!
 * jQuery jqCart Plugin v1.1.2
 * requires jQuery v1.9 or later
 *
 * http://incode.pro/
 *
 * Date: Date: 2016-05-18 19:15
 */
;(function ($) {
    'use strict';
    var cartData,
        itemData,
        orderPreview = '',
        totalCnt = 0,
        visibleLabel = false,
        label = $('<div class="jqcart-cart-label"><span class="jqcart-title">Оформить заказ</span><span class="jqcart-total-cnt">0</span></div>'),
        modal = '<div class="jqcart-layout"><div class="jqcart-checkout">123</div></div>',
        orderform = '<p class="jqcart-cart-title">Контактная информация:</p><form class="jqcart-orderform"><p><label>ФИО:</label><input type="text" name="user_name"></p><p><label>Телефон:</label><input type="text" name="user_phone"></p><p><label>Email:</label><input type="text" name="user_mail"></p><p><label>Адрес:</label><input type="text" name="user_address"></p><p><label>Коментарий:</label><textarea name="user_comment"></textarea></p><p><input type="submit" value="Отправить заказ"><input type="reset" value="Вернуться к покупкам"></p></form>';
    // Опции поумолчанию
    var opts = {
        buttons: '.add_item',
        cartLabel: 'body',
        visibleLabel: false,
        openByAdding: false,
        handler: '/',
        currency: '$'
    };
    /**
     * Закрытые ("приватные") методы для работы плагина
     */
    var actions = {
        /**
         * Инициализация плагина
         * @param o
         * @returns {boolean}
         */
        init: function (o) {
            // расширяем опции
            opts = $.extend(opts, o);
            // получаем данные из localStorage
            cartData = actions.getStorage();
            // если данные в корзине уже есть, то подсчитываем кол-во товаров для вывода на экран
            if (cartData !== null && Object.keys(cartData).length) {
                for (var key in cartData) {
                    if (cartData.hasOwnProperty(key)) {
                        totalCnt += cartData[key].count;
                    }
                }
                visibleLabel = true;
            }
            // Если товары есть или в опциях задано "всегда показывать",
            // то добавляем в ярлык кол-во товара и показываем
            // так же устанавливаем обработчик события click, при котором будет открываться корзина
            label.prependTo(opts.cartLabel)[visibleLabel || opts.visibleLabel ? 'show' : 'hide']()
                .on('click', actions.openCart)
                .find('.jqcart-total-cnt').text(totalCnt);

            // делегированно устанавливаем остальные обработчики события:
            // 1. клик на кнопку "Добавить"
            // 2. клик на слой под корзиной для закрытия
            // 3. клик на кнопки "+" и "-"
            // 4. изменение поля кол-ва в ручом режиме
            // 5. клик на кнопку "Удалить" товар из корзины
            // 6. отправка формы на сервер
            // 7. сброс данных формы
            // 8. клик вывода на печать
            $(document)
                .on('click', opts.buttons, actions.addToCart)
                .on('click', '.jqcart-layout', function (e) {
                    if (e.target === this) {
                        actions.hideCart();
                    }
                })
                .on('click', '.jqcart-incr', actions.changeAmount)
                .on('input keyup', '.jqcart-amount', actions.changeAmount)
                .on('click', '.jqcart-del-item', actions.delFromCart)
                .on('submit', '.jqcart-orderform', actions.sendOrder)
                .on('reset', '.jqcart-orderform', actions.hideCart)
                .on('click', '.jqcart-print-order', actions.printOrder);
            return false;
        },
        /**
         * Добавление товаров в корзину
         * @param e
         * @returns {boolean}
         */
        addToCart: function (e) {
            e.preventDefault();
            // все data-атрибуты кнопки
            itemData = $(this).data();
            if (typeof itemData.id === 'undefined') {
                console.log('Отсутствует ID товара');
                return false;
            }
            // данные из localStorage
            cartData = actions.getStorage() || {};
            // Если в корзине уже есть товар, то увеличиваем его кол-во на единицу
            // в ином случае - кол-во единица
            if (cartData.hasOwnProperty(itemData.id)) {
                cartData[itemData.id].count++;
            } else {
                itemData.count = 1;
                cartData[itemData.id] = itemData;
            }
            // Записываем данные в localStorage
            actions.setStorage(cartData);
            // изменяем кол-во в ярлыке
            actions.changeTotalCnt(1);
            label.show();
            // Если в опциях "открывать корзину сразу после добавления", то открываем ёё (ваш К.О.)
            if (opts.openByAdding) {
                actions.openCart();
            }
            return false;
        },
        /**
         * Удаление товара из корзины
         * @returns {boolean}
         */
        delFromCart: function () {
            // кнопка "Удалить" товар из корзины
            var $that = $(this),
            // родительский элемент кнопки (строка товара)
                line = $that.closest('.jqcart-tr'),
            // ID удаляемого товара
                itemId = line.data('id');
            // Данные из localStorage
            cartData = actions.getStorage();
            // изменяем общее кол-во в ярлыке
            actions.changeTotalCnt(-cartData[itemId].count);
            // удаляем товар из объекта
            delete cartData[itemId];
            // записываем объект обратно в localStorage
            actions.setStorage(cartData);
            // Удаляем строку с товаром
            line.remove();
            // пересчитываем сумму корзины
            actions.recalcSum();
            return false;
        },
        /**
         * Изменение количества товаров в корзине
         * @returns {boolean}
         */
        changeAmount: function () {
            // Элемент, на котором произошло событие: или текстовое поле, или кнопки "+", "-"
            var $that = $(this),
            // если элемент имеет класс 'jqcart-amount', то это текстовое поле и кол-во вводилось вручную
                manually = $that.hasClass('jqcart-amount'),
                // вводимое новое количество товара
                amount = +(manually ? $that.val() : $that.data('incr')),
                // ID товара
                itemId = $that.closest('.jqcart-tr').data('id');
            // данные из localStorage
            cartData = actions.getStorage();
            // Расчет количества с контролем "не меньше 1"
            // для кнопок "+" и "-" - вывод нового количества в текстовое поле
            if (manually) {
                cartData[itemId].count = isNaN(amount) || amount < 1 ? 1 : amount;
            } else {
                cartData[itemId].count += amount;
            }
            if (cartData[itemId].count < 1) {
                cartData[itemId].count = 1;
            }
            if (manually) {
                $that.val(cartData[itemId].count);
            } else {
                $that.siblings('input').val(cartData[itemId].count);
            }
            // Записываем данные с измененным количество в localStorage
            actions.setStorage(cartData);
            // Пересчитываем сумму корзины
            actions.recalcSum();
            return false;
        },
        /**
         * Пересчет суммы корзины
         * @returns {boolean}
         */
        recalcSum: function () {
            var subtotal = 0,
                amount,
                sum = 0,
                totalCnt = 0;
            // в цикле проходимся по каждой строке товара, умнозаем кол-во на цену,
            // плюс собираем общую сумму "Итого"
            $('.jqcart-tr').each(function () {
                amount = +$('.jqcart-amount', this).val();
                sum = Math.ceil((amount * $('.jqcart-price', this).text()) * 100) / 100;
                $('.jqcart-sum', this).html(sum + ' ' + opts.currency);
                subtotal = Math.ceil((subtotal + sum) * 100) / 100;
                totalCnt += amount;
            });
            // Выводим расчеты на экран
            $('.jqcart-subtotal strong').text(subtotal);
            $('.jqcart-total-cnt').text(totalCnt);
            // Если общее кол-во <= 0, то закрываем корзину
            if (totalCnt <= 0) {
                actions.hideCart();
                if (!opts.visibleLabel) {
                    label.hide();
                }
            }
            return false;
        },
        /**
         * Изменение количества товара, выводимое в ярлыке
         * @param n
         * @returns {boolean}
         */
        changeTotalCnt: function (n) {
            var cntOutput = $('.jqcart-total-cnt');
            cntOutput.text((+cntOutput.text() + n));
            return false;
        },
        /**
         * Открытие корзины
         * Тут, собственно, и комментировать нечего
         * Достали данные из localStorage и раскидываем по элементам
         */
        openCart: function () {
            var subtotal = 0,
                cartHtml = '';
            cartData = actions.getStorage();
            orderPreview = '<p class="jqcart-cart-title">Корзина <span class="jqcart-print-order"></span></p><div class="jqcart-table-wrapper"><div class="jqcart-manage-order"><div class="jqcart-thead"><div>ID</div><div></div><div>Наименование</div><div>Цена</div><div>Кол-во</div><div>Сумма</div><div></div></div>';
            var key, sum = 0;
            for (key in cartData) {
                if (cartData.hasOwnProperty(key)) {
                    sum = Math.ceil((cartData[key].count * cartData[key].price) * 100) / 100;
                    subtotal = Math.ceil((subtotal + sum) * 100) / 100;

                    orderPreview += '<div class="jqcart-tr" data-id="' + cartData[key].id + '">';
                    orderPreview += '<div class="jqcart-small-td">' + cartData[key].id + '</div>';
                    orderPreview += '<div class="jqcart-small-td jqcart-item-img"><img src="' + cartData[key].img + '" alt=""></div>';
                    orderPreview += '<div>' + cartData[key].title + '</div>';
                    orderPreview += '<div class="jqcart-price">' + cartData[key].price + '</div>';
                    orderPreview += '<div><span class="jqcart-incr" data-incr="-1">&#8211;</span><input type="text" class="jqcart-amount" value="' + cartData[key].count + '"><span class="jqcart-incr" data-incr="1">+</span></div>';
                    orderPreview += '<div class="jqcart-sum">' + sum + ' ' + opts.currency + '</div>';
                    orderPreview += '<div class="jqcart-small-td"><span class="jqcart-del-item"></span></div>';
                    orderPreview += '</div>';
                }
            }
            orderPreview += '</div></div>';
            orderPreview += '<div class="jqcart-subtotal">Итого: <strong>' + subtotal + '</strong> ' + opts.currency + '</div>';

            cartHtml = subtotal ? (orderPreview + orderform) : '<h2 class="jqcart-empty-cart">Корзина пуста</h2>';
            $(modal).appendTo('body').find('.jqcart-checkout').html(cartHtml);
        },
        /**
         * Закрытие корзины
         * @returns {boolean}
         */
        hideCart: function () {
            $('.jqcart-layout').fadeOut('fast', function () {
                $(this).remove();
            });
            return false;
        },
        /**
         * Отправка данных на сервер
         * @param e
         * @returns {boolean}
         */
        sendOrder: function (e) {
            e.preventDefault();
            var $that = $(this);
            // Проверка на пустые поля для имени и телефона юзера
            if ($.trim($('[name=user_name]', $that).val()) === '' || $.trim($('[name=user_phone]', $that).val()) === '') {
                $('<p class="jqcart-error">Пожалуйста, укажите свое имя и контактный телефон!</p>').insertBefore($that).delay(3000).fadeOut();
                return false;
            }
            // Ajax запрос на сервер
            $.ajax({
                url: opts.handler,
                type: 'POST',
                dataType: 'json',
                data: {
                    orderlist: $.param(actions.getStorage()),
                    userdata: $that.serialize()
                },
                error: function () {
                },
                success: function (resp) {
                    $('.jqcart-checkout').html('<p>' + resp.message + '</p>');
                    if (!resp.errors) {
                        setTimeout(methods.clearCart, 2000);
                    }
                }
            });
        },
        /**
         * Печать заказа
         * @returns {boolean}
         */
        printOrder: function () {
            var data = $(this).closest('.jqcart-checkout').prop('outerHTML');
            if (!data) {
                return false;
            }
            var win = window.open('', 'Печать заказа', 'width=' + screen.width + ',height=' + screen.height),
                cssHref = $(win.opener.document).find('link[href$="jqcart.css"]').attr('href'),
                d = new Date(),
                curDate = ('0' + d.getDate()).slice(-2) + '-' + ('0' + (d.getMonth() + 1)).slice(-2) + '-' + d.getFullYear() + ' ' + ('0' + d.getHours()).slice(-2) + ':' + ('0' + d.getMinutes()).slice(-2) + ':' + ('0' + d.getSeconds()).slice(-2);


            win.document.write('<html><head><title>Заказ ' + curDate + '</title>');
            win.document.write('<link rel="stylesheet" href="' + cssHref + '" type="text/css" />');
            win.document.write('</head><body >');
            win.document.write(data);
            win.document.write('</body></html>');

            win.document.close(); // нужно для IE >= 10
            win.focus(); // нужно для IE >= 10

            win.print();
            win.close();

            return true;
        },
        /**
         * Запись данных в localStorage
         * @param o
         * @returns {boolean}
         */
        setStorage: function (o) {
            localStorage.setItem('jqcart', JSON.stringify(o));
            return false;
        },
        /**
         * Получение данных из localStorage
         */
        getStorage: function () {
            return JSON.parse(localStorage.getItem('jqcart'));
        }
    };
    /**
     * Публичные методы
     */
    var methods = {
        /**
         * Очистка корзины
         */
        clearCart: function () {
            localStorage.removeItem('jqcart');
            label[opts.visibleLabel ? 'show' : 'hide']().find('.jqcart-total-cnt').text(0);
            actions.hideCart();
        },
        /**
         * Открытие корзины
         */
        openCart: actions.openCart,
        /**
         * Печать заказа
         */
        printOrder: actions.printOrder
    };
    /**
     * инициализация плагина
     * @param opts
     * @returns {*}
     */
    $.jqCart = function (opts) {
        if (methods[opts]) {
            return methods[opts].apply(this, Array.prototype.slice.call(arguments, 1));
        } else if (typeof opts === 'object' || !opts) {
            return actions.init.apply(this, arguments);
        } else {
            $.error('Метод с именем "' + opts + '" не существует!');
        }
    };
})(jQuery);

